// file      : structure-xsd.cxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2006 Code Synthesis Tools CC
// license   : GNU GPL v2; see accompanying LICENSE file

#include <iostream>
#include <fstream>

#include "time.hxx"
#include "structure.hxx"

using namespace std;

typedef
Attribute<void, void, void, void, void, void, void, void>
AttributeParser;

typedef
Element<void, void, void, void, void, void, void>
ElementParser;

typedef
Any<void, void, void, void, void, void>
AnyParser;

typedef
AllOptional<void, void, void, void, void>
AllOptionalParser;

typedef
AllRequired<void, void, void, void, void>
AllRequiredParser;

typedef
Choice<void, void, void, void, void, void, void, void>
ChoiceParser;

typedef
Sequence<void, void, void, void, void, void, void, void>
SequenceParser;

typedef
Empty<void, void, void>
EmptyParser;

typedef
EmptyExtension<void, void, void, void>
EmptyExtensionParser;

typedef
EmptyRestriction<void, void, void, void>
EmptyRestrictionParser;

typedef
Simple<void, void, void>
SimpleParser;

typedef
SimpleExtension<void, void, void, void>
SimpleExtensionParser;

typedef
SimpleRestriction<void, void, void, void>
SimpleRestrictionParser;

typedef
Complex<void, void, void, void, void, void, void>
ComplexParser;

typedef
ComplexExtension<void, void, void, void, void, void, void, void, void,
                 void, void, void>
ComplexExtensionParser;

typedef
ComplexRestriction<void, void, void, void, void, void, void, void, void,
                   void, void, void>
ComplexRestrictionParser;

typedef
Structure<void, void, void, void, void, void, void, void, void, void,
          void, void, void, void, void, void, void>
StructureParser;

int
main (int argc, char* argv[])
{
  if (argc != 2)
  {
    cerr << "usage: " << argv[0] << " strcuture.xml" << endl;
    return 1;
  }

  try
  {
    ifstream ifs;
    ifs.exceptions (ios_base::failbit);
    ifs.open (argv[1], ios::in | ios::ate);

    size_t size (ifs.tellg ());
    ifs.seekg (0, ios::beg);

    char* buf = new char[size];
    ifs.read (buf, size);

    cerr << "document size: " << size << " bytes" << endl;

    XML_Parser xml_parser (XML_ParserCreateNS (0, ' '));

    AttributeParser attribute;
    ElementParser element;
    AnyParser any;
    AllOptionalParser all_optional;
    AllRequiredParser all_required;
    ChoiceParser choice;
    SequenceParser sequence;
    EmptyParser empty_parser;
    EmptyExtensionParser empty_extension_parser;
    EmptyRestrictionParser empty_restriction_parser;
    SimpleParser simple_parser;
    SimpleExtensionParser simple_extension_parser;
    SimpleRestrictionParser simple_restriction_parser;
    ComplexParser complex_parser;
    ComplexExtensionParser complex_extension_parser;
    ComplexRestrictionParser complex_restriction_parser;
    StructureParser structure;

    structure.attribute_parser (attribute);
    structure.element_parser (element);
    structure.any_parser (any);
    structure.all_optional_parser (all_optional);
    structure.all_required_parser (all_required);
    structure.choice_parser (choice);
    structure.empty_parser (empty_parser);
    structure.empty_extension_parser (empty_extension_parser);
    structure.empty_restriction_parser (empty_restriction_parser);
    structure.simple_parser (simple_parser);
    structure.simple_extension_parser (simple_extension_parser);
    structure.simple_restriction_parser (simple_restriction_parser);
    structure.complex_parser (complex_parser);
    structure.complex_extension_parser (complex_extension_parser);
    structure.complex_restriction_parser (complex_restriction_parser);

    xml_schema::document<void> doc (
      structure,
      "http://www.codesynthesis.com/xmlns/xsdbench/structure",
      "structure");

    OS::Time start;

    for (unsigned long i (0); i < 10000; ++i)
    {
      doc.parse_begin (xml_parser);

      XML_Parse (xml_parser, buf, size, 1);

      doc.parse_end ();

      XML_ParserReset (xml_parser, 0);
    }

    OS::Time end;

    cerr << "time: " << end - start << endl;

    XML_ParserFree (xml_parser);

    delete[] buf;
  }
  catch (xml_schema::exception const& e)
  {
    cerr << e << endl;
    return 1;
  }
  catch (std::ios_base::failure const&)
  {
    cerr << "io failure" << endl;
    return 1;
  }
}
